@@ -87,15 +87,36 @@ Surface *Surface::fromWindow(QWindow *window)
     if (!native) {
         return nullptr;
     }
+    window->create();
     wl_surface *s = reinterpret_cast<wl_surface*>(native->nativeResourceForWindow(QByteArrayLiteral("surface"), window));
     if (!s) {
         return nullptr;
     }
+    if (auto surface = get(s)) {
+        return surface;
+    }
     Surface *surface = new Surface(window);
     surface->d->surface.setup(s, true);
     return surface;
 }
 
+Surface *Surface::fromQtWinId(WId wid)
+{
+    QWindow *window = nullptr;
+
+    for (auto win : qApp->allWindows()) {
+        if (win->winId() == wid) {
+            window = win;
+            break;
+        }
+    }
+
+    if (!window) {
+        return nullptr;
+    }
+    return fromWindow(window);
+}
+
 void Surface::release()
 {
     d->surface.release();
@@ -129,9 +150,11 @@ void Surface::Private::handleFrameCallback()
     emit q->frameRendered();
 }
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 const struct wl_callback_listener Surface::Private::s_listener = {
         frameCallback
 };
+#endif
 
 void Surface::Private::setupFrameCallback()
 {
